@@ -1,184 +1,60 @@
 package Games::Solitaire::BlackHole::Solver::Golf::App;
-$Games::Solitaire::BlackHole::Solver::Golf::App::VERSION = '0.2.2';
-use strict;
-use warnings;
-
+$Games::Solitaire::BlackHole::Solver::Golf::App::VERSION = '0.4.0';
 use 5.014;
+use Moo;
 
-use Getopt::Long;
-use Pod::Usage;
+extends('Games::Solitaire::BlackHole::Solver::App::Base');
+use Games::Solitaire::BlackHole::Solver::App::Base qw/ $card_re /;
 
 
-sub new
-{
-    my $class = shift;
-    return bless {}, $class;
-}
-
-my @ranks = ( "A", 2 .. 9, qw(T J Q K) );
-my %ranks_to_n = ( map { $ranks[$_] => $_ } 0 .. $#ranks );
-my $RANK_KING  = $ranks_to_n{'K'};
-
-my $card_re_str = '[' . join( "", @ranks ) . '][HSCD]';
-my $card_re     = qr{$card_re_str};
-
-sub _get_rank
-{
-    return $ranks_to_n{ substr( shift(), 0, 1 ) };
-}
-
-sub _calc_lines
-{
-    my $filename = shift;
-
-    if ( $filename eq "-" )
-    {
-        return [<STDIN>];
-    }
-    else
-    {
-        open my $in, "<", $filename
-            or die
-            "Could not open $filename for inputting the board lines - $!";
-        my @lines = <$in>;
-        close($in);
-        return \@lines;
-    }
-}
-
 sub run
 {
-    my $output_fn;
-
-    my ( $help, $man, $version );
+    my $self      = shift;
+    my $RANK_KING = $self->_RANK_KING;
 
     # A boolean
     my $place_queens_on_kings = '';
 
     # A boolean
     my $wrap_ranks = '';
-
-    GetOptions(
-        "o|output=s"       => \$output_fn,
-        "queens-on-kings!" => \$place_queens_on_kings,
-        "wrap-ranks!"      => \$wrap_ranks,
-        'help|h|?'         => \$help,
-        'man'              => \$man,
-        'version'          => \$version,
-    ) or pod2usage(2);
-
-    pod2usage(1) if $help;
-    pod2usage( -exitstatus => 0, -verbose => 2 ) if $man;
-
-    if ($version)
-    {
-        print
-"black-hole-solve version $Games::Solitaire::BlackHole::Solver::App::VERSION\n";
-        exit(0);
-    }
-
+    $self->_process_cmd_line(
+        {
+            extra_flags => {
+                "queens-on-kings!" => \$place_queens_on_kings,
+                "wrap-ranks!"      => \$wrap_ranks,
+            }
+        }
+    );
     if ($wrap_ranks)
     {
         $place_queens_on_kings = 1;
     }
 
-    my $filename = shift(@ARGV);
-
-    my $output_handle;
-
-    if ( defined($output_fn) )
-    {
-        open( $output_handle, ">", $output_fn )
-            or die "Could not open '$output_fn' for writing";
-    }
-    else
-    {
-        open( $output_handle, ">&STDOUT" );
-    }
-
-    my @lines = @{ _calc_lines($filename) };
-    chomp(@lines);
-
-    my $talon_line = shift(@lines);
-    my @talon_cards;
+    my $talon_line = shift( @{ $self->_board_lines } );
     my @talon_values;
     my $talon_ptr = 0;
     if ( my ($cards) = $talon_line =~ m{\ATalon:((?: $card_re){16})\z} )
     {
-        @talon_cards  = $cards =~ /($card_re)/g;
-        @talon_values = map { _get_rank($_) } @talon_cards;
+        @talon_values = map { $self->_get_rank($_) }
+            @{ $self->_talon_cards( [ $cards =~ /($card_re)/g ] ) };
     }
     else
     {
         die "Could not match first talon line!";
     }
-    my $found_line = shift(@lines);
 
-    my $init_foundation;
-    if ( my ($card) = $found_line =~ m{\AFoundations: ($card_re)\z} )
-    {
-        $init_foundation = _get_rank($card);
-    }
-    else
-    {
-        die "Could not match first foundation line!";
-    }
+    $self->_set_up_solver( $talon_ptr,
+        [ 1, ( $wrap_ranks ? ($RANK_KING) : () ) ] );
 
-    my @board_cards  = map { [ split /\s+/, $_ ] } @lines;
-    my @board_values = map {
-        [ map { _get_rank($_) } @$_ ]
-    } @board_cards;
-
-    my $init_state = "";
-
-    vec( $init_state, 0, 8 ) = $init_foundation;
-    vec( $init_state, 1, 8 ) = $talon_ptr;
-
-    foreach my $col_idx ( 0 .. $#board_values )
-    {
-        vec( $init_state, 4 + $col_idx, 4 ) =
-            scalar( @{ $board_values[$col_idx] } );
-    }
-
-    # The values of %positions is an array reference with the 0th key being the
-    # previous state, and the 1th key being the column of the move.
-    my %positions = ( $init_state => [] );
-
-    my @queue = ($init_state);
-
-    my %is_good_diff =
-        ( map { $_ => 1 }
-            ( (-1), 1, ( $wrap_ranks ? ( ( -$RANK_KING ), $RANK_KING ) : () ) )
-        );
+    my $positions    = $self->_positions;
+    my $board_values = $self->_board_values;
 
     my $verdict = 0;
 
-    my $trace_solution = sub {
-        my $final_state = shift;
-
-        my $state = $final_state;
-        my ( $prev_state, $col_idx );
-
-        my @moves;
-        while ( ( $prev_state, $col_idx ) = @{ $positions{$state} } )
-        {
-            push @moves,
-                (
-                ( $col_idx == @board_cards )
-                ? "Deal talon " . $talon_cards[ vec( $prev_state, 1, 8 ) ]
-                : $board_cards[$col_idx]
-                    [ vec( $prev_state, 4 + $col_idx, 4 ) - 1 ]
-                );
-        }
-        continue
-        {
-            $state = $prev_state;
-        }
-        print {$output_handle} map { "$_\n" } reverse(@moves);
-    };
+    $self->_next_task;
 
 QUEUE_LOOP:
-    while ( my $state = pop(@queue) )
+    while ( my $state = $self->_get_next_state_wrapper )
     {
         # The foundation
         my $fnd      = vec( $state, 0, 8 );
@@ -186,38 +62,16 @@ QUEUE_LOOP:
         my $tln      = vec( $state, 1, 8 );
         my @sub_queue;
 
+        my @_pending;
+
         if ( $place_queens_on_kings || ( $fnd != $RANK_KING ) )
         {
-            # my @debug_pos;
-            foreach my $col_idx ( 0 .. $#board_values )
-            {
-                my $pos = vec( $state, 4 + $col_idx, 4 );
-
-                # push @debug_pos, $pos;
-                if ($pos)
-                {
-                    $no_cards = 0;
-
-                    my $card = $board_values[$col_idx][ $pos - 1 ];
-                    if ( exists( $is_good_diff{ ( $card - $fnd ) } ) )
-                    {
-                        my $next_s = $state;
-                        vec( $next_s, 0, 8 ) = $card;
-                        --vec( $next_s, 4 + $col_idx, 4 );
-                        if ( !exists( $positions{$next_s} ) )
-                        {
-                            # print "$card $fnd $col_idx\n";
-                            $positions{$next_s} = [ $state, $col_idx ];
-                            push( @sub_queue, $next_s );
-                        }
-                    }
-                }
-            }
+            $self->_find_moves( \@sub_queue, $state, \$no_cards );
         }
         else
         {
         COL:
-            foreach my $col_idx ( 0 .. $#board_values )
+            foreach my $col_idx ( keys @$board_values )
             {
                 my $pos = vec( $state, 4 + $col_idx, 4 );
 
@@ -229,41 +83,33 @@ QUEUE_LOOP:
             }
         }
 
-        # print "Checking ", join(",", @debug_pos), "\n";
         if ($no_cards)
         {
-            print {$output_handle} "Solved!\n";
-            $trace_solution->($state);
+            $self->_trace_solution( $state, );
             $verdict = 1;
             last QUEUE_LOOP;
         }
-        elsif ( $tln < @talon_values )
+
+        if ( $tln < @talon_values )
         {
             my $next_s = $state;
             vec( $next_s, 0, 8 ) = $talon_values[$tln];
             ++vec( $next_s, 1, 8 );
-            if ( !exists( $positions{$next_s} ) )
+            if ( !exists( $positions->{$next_s} ) )
             {
-                $positions{$next_s} = [ $state, scalar(@board_values) ];
-                push( @queue, $next_s );
+                $positions->{$next_s} =
+                    [ $state, scalar(@$board_values), 1, 0 ];
+                push @_pending, [ $next_s, 0 ];
             }
         }
 
         # Give preference to non-talon moves
-        push @queue, @sub_queue;
+        push @_pending, @sub_queue;
+        last QUEUE_LOOP
+            if not $self->_process_pending_items( \@_pending, $state );
     }
 
-    if ( !$verdict )
-    {
-        print {$output_handle} "Unsolved!\n";
-    }
-
-    if ( defined($output_fn) )
-    {
-        close($output_handle);
-    }
-
-    exit( !$verdict );
+    return $self->_my_exit( $verdict, );
 }
 
 
@@ -282,7 +128,7 @@ implemented as a class to solve the “Golf” variant of solitaire.
 
 =head1 VERSION
 
-version 0.2.2
+version 0.4.0
 
 =head1 SYNOPSIS
 
@@ -351,10 +197,6 @@ More information about Golf Solitaire can be found at:
 
 =back
 
-=head1 VERSION
-
-version 0.2.2
-
 =head1 METHODS
 
 =head2 $self->new()
@@ -409,28 +251,7 @@ WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-=head1 AUTHOR
-
-Shlomi Fish <shlomif@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2010 by Shlomi Fish.
-
-This is free software, licensed under:
-
-  The MIT (X11) License
-
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website
-L<https://github.com/shlomif/games-solitaire-blackhole-solver/issues>
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
-=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+=for :stopwords cpan testmatrix url bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 SUPPORT
 
@@ -451,14 +272,6 @@ L<https://metacpan.org/release/Games-Solitaire-BlackHole-Solver>
 
 =item *
 
-Search CPAN
-
-The default CPAN search engine, useful to view POD in HTML format.
-
-L<http://search.cpan.org/dist/Games-Solitaire-BlackHole-Solver>
-
-=item *
-
 RT: CPAN's Bug Tracker
 
 The RT ( Request Tracker ) website is the default bug/issue tracking system for CPAN.
@@ -467,22 +280,6 @@ L<https://rt.cpan.org/Public/Dist/Display.html?Name=Games-Solitaire-BlackHole-So
 
 =item *
 
-AnnoCPAN
-
-The AnnoCPAN is a website that allows community annotations of Perl module documentation.
-
-L<http://annocpan.org/dist/Games-Solitaire-BlackHole-Solver>
-
-=item *
-
-CPAN Ratings
-
-The CPAN Ratings is a website that allows community ratings and reviews of Perl modules.
-
-L<http://cpanratings.perl.org/d/Games-Solitaire-BlackHole-Solver>
-
-=item *
-
 CPANTS
 
 The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
@@ -531,4 +328,25 @@ L<https://github.com/shlomif/black-hole-solitaire>
 
   git clone https://github.com/shlomif/black-hole-solitaire
 
+=head1 AUTHOR
+
+Shlomi Fish <shlomif@cpan.org>
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+L<https://github.com/shlomif/games-solitaire-blackhole-solver/issues>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2010 by Shlomi Fish.
+
+This is free software, licensed under:
+
+  The MIT (X11) License
+
 =cut
